################################################################
# Tests for bootstrap CI Helper Functions.
# 2014 Pierre Dragicevic
################################################################

# Copy & paste pieces of code below.

#############################################################

setwd("~/Desktop/BetterRitual/R code")
source("bootstrap_macros.R")

############################################################
# Basic test
############################################################
set.seed(0)
data <- rnorm(12, mean=3, sd=2)
mean_with_ci <- bootstrapMeanCI.text(data, "s")
cat("Mean is", mean_with_ci, "\n")
median_with_ci <- bootstrapMedianCI.text(data, "s")
cat("Median is", median_with_ci, "\n")

set.seed(4)
data <- rnorm(12, mean=30, sd=2)
mean_with_ci <- bootstrapMeanCI.text(data, "s")
cat("Mean is", mean_with_ci, "\n")
median_with_ci <- bootstrapMedianCI.text(data, "s")
cat("Median is", median_with_ci, "\n")

############################################################
# Comparison of bootstrap vs. exact CIs
############################################################

#samples <- 10
#tests <- 200
#lim <- 2

#samples <- 100
#tests <- 200
#lim <- 0.5

#samples <- 500
#tests <- 200
#lim <- 0.2

#samples <- 5
#tests <- 1000
#lim <- 3

samples <- 10
tests <- 200
lim <- 1

set.seed(0)
replicates <- 1000

lowerBounds.e <- NULL
lowerBounds.b <- NULL
upperBounds.e <- NULL
upperBounds.b <- NULL
for (i in 0:tests) {
	cat(i, "\n")
	set.seed(i)
	data <- rnorm(samples, mean=0, sd=1)
	mean_with_exact_ci <- exactMeanCI(data)
	mean_with_bootstrap_ci <- bootstrapMeanCI(data)
	lowerBounds.e <- c(lowerBounds.e, mean_with_exact_ci[2])
	lowerBounds.b <- c(lowerBounds.b, mean_with_bootstrap_ci[2])
	upperBounds.e <- c(upperBounds.e, mean_with_exact_ci[3])
	upperBounds.b <- c(upperBounds.b, mean_with_bootstrap_ci[3])
}

######## Comparison of lower and upper CI bounds

plot(lowerBounds.e, lowerBounds.b, 
     panel.first = lines(stats::lowess(lowerBounds.e, lowerBounds.b), lty = "dashed"),
     pch = 0, cex = 0.5, xlim = c(-lim, lim), ylim = c(-lim, lim),
     col = "blue")

points(upperBounds.e, upperBounds.b,
     panel.first = lines(stats::lowess(upperBounds.e, upperBounds.b), lty = "dashed"),
     pch = 0, cex = 0.5, xlim = c(-lim, lim), ylim = c(-lim, lim),
	 col = "red", frame.plot=F)

abline(0, 1)

######### Comparison of CI length

plot(upperBounds.e - lowerBounds.e, upperBounds.b - lowerBounds.b, 
     panel.first = lines(stats::lowess(upperBounds.e - lowerBounds.e, upperBounds.b - lowerBounds.b), lty = "dashed"),
     pch = 0, cex = 0.5, xlim = c(0, 3), ylim = c(0, 3),
     col = "blue")
abline(0, 1)


############################################################
# Data from ??
############################################################
lizard <- c(6.2, 6.6, 7.1, 7.4, 7.6, 7.9, 8, 8.3, 8.4, 8.5, 8.6, 8.8, 8.8, 9.1, 9.2, 9.4, 9.4, 9.7, 9.9, 10.2, 10.4, 10.8, 11.3, 11.9)
t.test(lizard)

############################################################
# Discrete data problem
############################################################
data <- c(1, 2, 2, 2, 2, 2, 2, 2, 3, 3, 4, 4, 5)
#data <- c(0.03846154, 0.01250000, 0.00000000, 0.00000000, 0.01234568, 0.00000000, 0.01234568, 0.00000000, 0.01250000, 0.00000000, 0.00000000, 0.00000000)
# This line adds a random perturbation. Boot does not work
# otherwise.
data <- data + runif(length(data), -1e-10, 1e-10)
median_with_cis <- bootstrapMedianCI.text(data, "")
cat("Median is", median_with_cis, "\n")

############################################################
# Discrete data problem
# (reformulated for newsgroup post)
############################################################

# Load package boot
library(boot)

# Make the code deterministic
set.seed(0)

# Define the median as the statistic of interest
samplemedian <- function(x, d) {return(median(x[d]))}

# Use many replicates
replicates <- 10000

# A problematic dataset
data1 <- c(1, 2, 2, 2, 2, 2, 2, 2, 3, 3, 4, 4, 5)

# The point estimate is 2
samplemedian(data1)

# boot.ci issues the following error:
#    Error in bca.ci(boot.out, conf, index[1L], L = L, t = t.o, t0 = t0.o,  : 
#      estimated adjustment 'w' is infinite
b <- boot(data1, samplemedian, R = replicates)
boot.ci(b, type = "bca")

# Add a negligible perturbation to the data
epsilon <- 1e-10
data1 <- data1 + runif(length(data1), -epsilon, epsilon)

# boot.ci now returns (2, 3) as the BCa confidence interval
b <- boot(data1, samplemedian, R = replicates)
boot.ci(b, type = "bca")

# Another problematic dataset
data2 <- c(1, 1, 2, 2, 2, 2, 2, 2, 2, 3, 3, 4, 4, 5)

# The point estimate is 2
samplemedian(data2)

# boot.ci returns (1, 1) and issues the following warnings:
#    Warning : BCa Intervals used Extreme Quantiles
#    Some BCa intervals may be unstable
#    Warning message:
#    In norm.inter(t, adj.alpha) : extreme order statistics used as endpoints
b <- boot(data2, samplemedian, R = replicates)
boot.ci(b, type = "bca")

# Add a negligible perturbation to the data
epsilon <- 1e-10
data2 <- data2 + runif(length(data2), -epsilon, epsilon)

# boot.ci now returns (2, 3) as the BCa confidence interval
b <- boot(data2, samplemedian, R = replicates)
boot.ci(b, type = "bca")

#########################################

