

# A SIMPLE SET OF PLOTTING FUNCTIONS

### Horizontal bars with error bars (WW) ###
hplot <- function(analysisData, title, datasetName, x, ymin, ymax, ylabelFormatter, width=4, height=1.5, facets=NULL) {
	library(ggplot2)
	library(grid)
	library(scales)

	dev.new(width=width, height=height)
	
	if(is.null(facets) == FALSE){
		plot <- ggplot(analysisData, aes_string(x=x, y="pointEstimate")) + facet_grid(facets)
	}
	else{
		plot <- ggplot(analysisData, aes_string(x=x, y="pointEstimate"))
	} 

	plot + geom_bar(width=0.7, fill="#00000033", stat="identity") +
		coord_flip() +
		geom_pointrange(aes(ymin=ci.min, ymax=ci.max), size=0.7) +
		geom_point(size=3) +
		theme_bw() + 
		theme( # remove the vertical grid lines
			plot.background = element_blank(),
			legend.position = "none",
			axis.ticks.margin=unit(c(0.5,-4.0),'mm'),
			axis.title.x = element_text(size=8, vjust=-0.5),
			axis.text.x = element_text(size=6.75, colour="#888888"),
			axis.text.y = element_text(size=8, colour="#000000FF", face="bold"),
	   	panel.border = element_blank(),
        	panel.grid.major.y = element_blank(),
        	panel.grid.minor.y = element_blank(),
        	axis.line = element_line(size=0.1, color="#000000"),
        	axis.ticks = element_line(size=0.1, color="#000000"),
        	axis.line.y = element_blank(),
        	axis.ticks.y = element_blank(),
        	panel.grid.minor.x = element_line(size=0.01, color="#000000"),
        	panel.grid.major.x = element_line(size=0.03, color="#000000"),
        	strip.background = element_rect(colour="#EEEEEE", fill="#EEEEEE"),
        	strip.text.y = element_text(size=7, face="bold"),
        	strip.text.x = element_text(size=8)
	    ) +
	    scale_x_discrete(name="") +
	    scale_y_continuous(name=title, limits = c(ymin, ymax), oob = rescale_none, labels=ylabelFormatter)
}

# Example Call
#hplot(expertiseTechDiffAnalysis, "Pairwise Change in Accuracy", datasetName, "techs", -0.1, 0.3, score_formatter, height=1.5, facets=(expertise ~ .))


### Vertical Bars w/ Color and a layout grid (PD) ###
plotResults <- function(analysisData, title, datasetName, ymin, ymax, ylabelFormatter, x="tech", facets=(. ~ map)) {
	library(ggplot2)
	library(grid)
	library(scales)

	# Right size for 3 bars
	#dev.new(width=2.2, height=5)
	# Right size for 3x3 bars
	dev.new(width=5.5, height=6)

	techcolors <- c("#9999FFFF", "#CC2222FF", "#550066FF")
	names(techcolors) <- levels(analysisData$tech)
	techcolors_faint <- c("#5555FF22", "#CC222233", "#55006633")
	names(techcolors_faint) <- levels(analysisData$tech)

	ggplot(analysisData, aes_string(x=x, y="pointEstimate", color=x, fill=x)) + facet_grid(facets) +
		geom_bar(width=0.75, stat="identity", colour="#FFFFFF00") +
		scale_colour_manual(name = "grp", values = techcolors) +
		scale_fill_manual(name = "grp", values = techcolors_faint) +
		geom_pointrange(aes(ymin=ci.min, ymax=ci.max), size=0.7) +
		geom_point(size=1.5) +
		theme_bw() + 
		theme( # remove the vertical grid lines
			plot.background = element_blank(),
			legend.position = "none",
			axis.ticks.margin=unit(c(-4,2),'mm'),
			axis.text.y = element_text(size=8.75, colour="#888888"),
			axis.text.x = element_text(size=9, colour=techcolors, face="bold"),
	   		panel.border = element_blank(),
	        panel.grid.major.x = element_blank(),
	        panel.grid.minor.y = element_blank(),
	        axis.ticks = element_blank(),
	        #panel.grid.minor.y = element_line(size=0.02, color="#000000"),
	        panel.grid.major.y = element_line(size=0.02, color="#000000"),
	        plot.title = element_text(size=12, vjust=2),
	        strip.text.x = element_text(colour = "white", face="bold")
	    ) +
	    scale_x_discrete(name="") +
	    scale_y_continuous(name="", limits = c(ymin, ymax), oob = rescale_none, labels=ylabelFormatter) +
	    ggtitle(paste(title, "for", datasetName))
}
